import 'dart:io';
import 'package:connectivity_plus/connectivity_plus.dart';
import 'package:device_info_plus/device_info_plus.dart';
import 'package:package_info_plus/package_info_plus.dart';

class DeviceUtility {
  DeviceUtility._();

  static Future<MyDeviceInfo> getInfo() async {
    ConnectivityResult connectivityResult = await Connectivity().checkConnectivity();
    DeviceInfoPlugin deviceInfo = DeviceInfoPlugin();
    PackageInfo packageInfo = await PackageInfo.fromPlatform();
    String company = "Unknown";
    String deviceModel = "Unknown";
    String osVersion = "Unknown";
    String versionNumber = packageInfo.version.split(".").sublist(0, 2).join(".");
    String deviceKey = "Unknown";
    String deviceType = "Unknown";
    if (Platform.isAndroid) {
      AndroidDeviceInfo androidInfo = await deviceInfo.androidInfo;
      company = androidInfo.manufacturer.toString().toUpperCase();
      deviceModel = androidInfo.model.toString().toUpperCase();
      osVersion = androidInfo.version.release!;
      deviceKey = androidInfo.id!;
      deviceType = "Android";
    } else if (Platform.isIOS){
      IosDeviceInfo iosInfo = await deviceInfo.iosInfo;
      company = "APPLE";
      deviceModel = iosInfo.utsname.machine.toString().toUpperCase();
      osVersion = iosInfo.systemVersion!;
      deviceKey = iosInfo.identifierForVendor!;
      deviceType = "Iphone";
    } else if (Platform.isWindows){
      WindowsDeviceInfo windowsInfo = await deviceInfo.windowsInfo;
      company = "Microsoft";
      deviceModel = "Unknown";
      osVersion = "Unknown";
      deviceKey = windowsInfo.computerName;
      deviceType = "Windows";
    }else if(Platform.isMacOS){
      MacOsDeviceInfo macInfo = await deviceInfo.macOsInfo;
      company = "APPLE";
      deviceModel = macInfo.model;
      osVersion = macInfo.osRelease;
      deviceKey = macInfo.computerName;
      deviceType = "Mac";
    }

    return MyDeviceInfo(
        company: company,
        deviceModel: deviceModel,
        osVersion: osVersion,
        deviceType: deviceType,
        deviceKey: deviceKey,
        versionNumber: versionNumber
    );





  }

}

class MyDeviceInfo {
  final String company;
  final String deviceModel;
  final String osVersion;
  final String versionNumber;
  final String deviceKey;
  final String deviceType;

  MyDeviceInfo({
    this.company="Unknown",
    this.deviceModel="Unknown",
    this.osVersion="Unknown",
    this.versionNumber="Unknown",
    this.deviceKey="Unknown",
    this.deviceType="Unknown",
  });


  Map<String, dynamic> toJson() => {
    "Company": company,
    "DeviceModel": deviceModel,
    "OsVersion": osVersion,
    "VersionNumber": versionNumber,
    "DeviceKey": deviceKey,
    "DeviceType": deviceType,
  };
}
